/******************************************************************
 * Physics models interactions demo
 *
 * Demonstrates between 2 box models: 
 *  - Prismatic Constraint
 *  - Stiff Spring Constraint 
 *  -
 *  - 
 *  -
 *  
 ******************************************************************/


// First of all, we have to add 2 boxes to the stage. 
boxa = SceneGroup("./simpleboxa.atmo").add();
boxb = SceneGroup("./simpleboxb.atmo").add();


// An Array to hold all the current constraint and action objects - 
// there can be more than one constraint and/or applied to a pair of objects

constraints = [];

actions = [];


// The FastConstraintSolver is the main "creator" of the physics 
// interactions in the javascript. 
// Read more about it here:
// http://www.adobe.com/products/atmosphere/support/javascriptapi/FastConstraintSolver.html
// 
// In this demo, we only use a single FastConstraintSolver to apply
// physics to 2 boxes. In reality, for more complex systems with many models, 
// we may want to have many FastConstraintSolvers to combine seperate sections
// models together. 
solver = FastConstraintSolver();



generalStrength = 0.9;
generalTau = 0.9;



// ************************************************************************
// Next, we set up the physical models stuff for each of the boxes
// ************************************************************************
boxa.reset = function() {
	if(this.getPhysicalModel()!=undefined) {
		this.deletePhysicalModel();
	}
	this.position = Vector(2, 5,-2);
	this.orientation = Rotation();
	this.createPhysicalModel("10");
	// Make it stationary
	this.getPhysicalModel().worldSpaceVelocity = Vector(0,0,0);
	this.getPhysicalModel().worldSpaceAcceleration = Vector(0,0,0);
	this.getPhysicalModel().worldSpaceAngularVelocity = Vector(0,0,0);
	this.getPhysicalModel().fixed= false;
	this.getPhysicalModel().collide= true;
}
boxa.onLoad = function() {

	// Redo the physical model setup.... 
	this.reset();	
	boxa.onLoad = undefined;
	removeAnimator(this);

}
addAnimator(boxa);

boxb.reset = function() {
	if(this.getPhysicalModel()!=undefined) {
		this.deletePhysicalModel();
	}
	this.position = Vector(-2, 5,-2);
	this.orientation = Rotation();
	this.createPhysicalModel("10");
	// Make it stationary
	this.getPhysicalModel().worldSpaceVelocity = Vector(0,0,0);
	this.getPhysicalModel().worldSpaceAcceleration = Vector(0,0,0);
	this.getPhysicalModel().worldSpaceAngularVelocity = Vector(0,0,0);
	this.getPhysicalModel().fixed= false;
	this.getPhysicalModel().collide= true;
}
boxb.onLoad = function() {
	
	// Redo the physical model setup.... 
	this.reset();

	boxb.onLoad = undefined;
	removeAnimator(this);

}
addAnimator(boxb);







// *******************************************************************************
// Once each of the physical models are set up for the boxes, 
// we can now add phyics constraint or action to the pair of models :)
//
// Each of these methods is for a single constraint/action, 
// and takes 3 parameters: 
//  - moda   : the first Model/SceneGroup 
//  - modb   : the second Model/SceneGroup
//  - solver : the FastConstraintSolver which is used to apply the constraint.
// 
// You can reuse these methods in your own programs... 
// ********************************************************************************
allowfixbox = true;


// -------------- Prismatic constraint ------------------------------------------

this.doPrismaticConstraint = function(moda, modb, solver) {

	// To demonstrate the Prismatic Constraint, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;

	
	// Next, we create a new PrismaticConstraint from the FastConstraintSolver
	var currentConstraint = solver.createPrismaticConstraint(moda.getPhysicalModel(), modb.getPhysicalModel()); 

	
	// Adjust the PrismaticConstraint variables.... 
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/PrismaticConstraint.html
	// For more details
	currentConstraint.strength = generalStrength;
	currentConstraint.tau = generalTau;
	currentConstraint.slideDirection = Vector(1.0,0,0);


	// Finally we must remember to ADD the constraint to the solver!
	solver.addConstraint(currentConstraint);
	
	
	// Ignore this bit...... ;)  Just adding the constraint to an Array
	constraints.push(currentConstraint);

}


// -------------- Stiff spring constraint ------------------------------------------

this.doStiffSpringConstraint = function(moda, modb, solver) {

	// To demonstrate the Stiff Spring Constraint, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;
	
	// Next, we create a new StiffSpringConstraint from the FastConstraintSolver
	var currentConstraint = solver.createStiffSpringConstraint(moda.getPhysicalModel(), modb.getPhysicalModel()); 
	
	// Adjust the StiffSpringConstraint...
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/StiffSpringConstraint.html
	// For more details
	currentConstraint.strength = generalStrength;
	currentConstraint.tau = generalTau;
	currentConstraint.length = 3.0;

	
	// Finally we must remember to ADD the constraint to the solver!
	solver.addConstraint(currentConstraint);

	// Ignore this bit...... ;)  Just adding the constraint to an Array
	constraints.push(currentConstraint);

}


// -----------  Point to Point constraint ------------------------------------------

this.doPointToPointConstraint = function(moda, modb, solver) {

	// To demonstrate the Point To Point Constraint, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;
	
	// Next, we create a new PointToPointConstraint from the FastConstraintSolver
	var currentConstraint = solver.createPointToPointConstraint(moda.getPhysicalModel(), modb.getPhysicalModel()); 
	
	// Adjust the PointToPointConstraint...
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/PointToPointConstraint.html
	// For more details
	currentConstraint.strength = generalStrength;
	currentConstraint.tau = generalTau;

	
	// Finally we must remember to ADD the constraint to the solver!
	solver.addConstraint(currentConstraint);

	// Ignore this bit...... ;)  Just adding the constraint to an Array
	constraints.push(currentConstraint);

}

// ---------  Angular Dashpot constraint ------------------------------------------

this.doAngularDashpot = function(moda, modb, solver) {

	// To demonstrate the Angular Dashpot, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;
	
	// Next, we create a new AngularDashpot from the FastConstraintSolver
	var currentConstraint = solver.createAngularDashpot(moda.getPhysicalModel(), modb.getPhysicalModel()); 
	
	// Adjust the AngularDashpot...
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/AngularDashpot.html
	// For more details
	currentConstraint.strength = generalStrength;
	
	// We set the damping at the default tau - please note, these
	// are seperate things! :)
	currentConstraint.damping = generalTau;

	
	// Finally we must remember to ADD the constraint to the solver!
	solver.addAngularDashpot(currentConstraint);

	// Ignore this bit...... ;)  Just adding the constraint to an Array
	constraints.push(currentConstraint);

}


// -------------- Hinge constraint ------------------------------------------

this.doHingeConstraint = function(moda, modb, solver) {

	// To demonstrate the Hinge Constraint, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;
	
	// Next, we create a new HingeConstraint from the FastConstraintSolver
	var currentConstraint = solver.createHingeConstraint(moda.getPhysicalModel(), modb.getPhysicalModel()); 
	
	// Adjust the HingeConstraint...
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/PointToPointConstraint.html
	// For more details
	currentConstraint.strength = generalStrength;
	currentConstraint.tau = generalTau;

	// Important to note! 
	// You must specify both the pivot and the axis point. 
	// The Pivot is the point around which the object will rotate
	// The Axis is the direction the pivot point that the object rotates
	// eg. if Axis == Vector(0, 1, 0), the object will rotate around the Y axis. 

	currentConstraint.setWorldSpacePivotAndAxis(Vector(0, 0, 0), Vector(0, 1, 0));


	
	// Finally we must remember to ADD the constraint to the solver!
	solver.addConstraint(currentConstraint);

	// Ignore this bit...... ;)  Just adding the constraint to an Array
	constraints.push(currentConstraint);

}


// -------------- Magnetic Action ------------------------------------------


this.doMagneticAction = function(moda, modb, solver) {

	// To demonstrate the Magnetic Action, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;



	// Next, we specify 6 variables to create the MagneticAction...

	var magneticStrength  = 100;		// determines the attrictive force
						// > 0 == positive attriction, < 0 == negative attraction
	var magneticFalloff   = 0;		// rate at which magnetism decreases
	var magneticAngle     = Math.pi * 2.0;  // radians. Angle of the magnetic field. Set to 360 degrees.
	var maximumDistance   = 100;   		// feet. 
	var magneticDirection = Vector(1, 0, 0);// only applies if magneticAngle is < 360 degrees.


	// we need to specify where we want the field to eminate from...	
	// for this demo, we put it in the same position as the "modb". 
	var magneticPosition  = modb.position;



	// We now create a new MagneticAction from the FastConstraintSolver
	// This will be our magnetic source from which eminates the magnetic field. 
	// http://www.adobe.com/products/atmosphere/support/javascriptapi/MagneticAction.html
	// For more details
	var currentAction = solver.createMagneticAction(magneticStrength,magneticFalloff,
				magneticAngle, maximumDistance, magneticPosition,magneticDirection); 	
	


	// Add a model which we want to be attracted to the Magnetic source. 
	currentAction.addPhysicalModel(moda.getPhysicalModel());
		
	// Finally we must remember to ADD the action to the solver!
	solver.addAction(currentAction);

	// Ignore this bit...... ;)  Just adding the action to an Array
	actions.push(currentAction);

}



this.doSimpleRigidWindAction = function(moda, modb, solver) {

	// To demonstrate the Simple Rigid Wind action, we first set 
	// a box to be at a fixed position	
	if(allowfixbox) 
	modb.getPhysicalModel().fixed = true;


	// http://www.adobe.com/products/atmosphere/support/javascriptapi/SimpleRigidWindAction.html
	// For more details
	
	var windDirection  = Vector(0, 0, 1.0);	// Where the wind blows
	var windStrength   = 5;				// Wind strength from 0 - 100.

	
	// Important to note - we need to create a SimpleRigidWindAction object 
	// for each object we want affected by the wind. So here, we create two
	// action objects, one for each "moda" and "modb".

	var currentAction = solver.createSimpleRigidWindAction(moda.getPhysicalModel(), windDirection, windStrength); 
	
	currentAction.active = true;
	
	// add the action to the solver!
	solver.addAction(currentAction);

	// ignore this
	actions.push(currentAction);
	
	// create the action for "modb"....
	currentAction = solver.createSimpleRigidWindAction(modb.getPhysicalModel(), windDirection, windStrength); 

	currentAction.active = true;	

	// add the action to the solver!
	solver.addAction(currentAction);
	
	// ignore this
	actions.push(currentAction);


	// there seems to be a bit of a bug where the wind doesn't affect something
	// which isn't already moving... so we peturb the object just a little
	// to cheat it :)

}



// ****************************************************************
// Helpfull little routine to remove all the constraints or actions
// currently set between the objects for a single FastConstraintSolver
// ****************************************************************
this.clearConstraints = function() {
	if(constraints.length > 0) {
		boxa.getPhysicalModel().fixed = false;
		boxb.getPhysicalModel().fixed = false;
		
		// Remove and delete each of the constraints 
		// in the array. 
		for(var i=constraints.length-1; i > -1; i--) {
			if(constraints[i] instanceof AngularDashpot) {
				solver.removeAngularDashpot(constraints[i]);
			} else {
				solver.removeConstraint(constraints[i]);
			}
			var cs = constraints.pop();
			delete cs;
		}
	}
}

this.clearActions = function() {
	if(actions.length > 0) {
		boxa.getPhysicalModel().fixed = false;
		boxb.getPhysicalModel().fixed = false;
		
		// Remove and delete each of the actions in the array
		for(var i=actions.length-1; i > -1; i--) {			
			solver.removeAction(actions[i]);
			var cs = constraints.pop();
			delete cs;
		}
		
	}
}





// *****************************************************************
// All the Buttons set up in the control panel go here :) 
// Not much to see here, move along!
// *****************************************************************

this.fixbox = Toggle("Have a box fixed to a position").add();
this.fixbox.state = allowfixbox;
this.fixbox.onClick = function(state) {	
	allowfixbox = state;
	boxb.getPhysicalModel().fixed = state;
}

this.setLowStr = Button("Low strength (0.1)").add();
this.setLowStr.onClick = function() {
	setStr(0.1);	
}
this.setMedStr = Button("Med strength (0.5)").add();
this.setMedStr.onClick = function() {
	setStr(0.5);	
}
this.setHighStr = Button("High strength (0.9)").add();
this.setHighStr.onClick = function() {
	setStr(0.9);	
}
this.setLowTau = Button("Low tau (0.1)").add();
this.setLowTau.onClick = function() {
	setTau(0.1);	
}
this.setMedTau = Button("Med tau (0.5)").add();
this.setMedTau.onClick = function() {
	setTau(0.5);	
}
this.setHighTau = Button("High tau (0.9)").add();
this.setHighTau.onClick = function() {
	setTau(0.9);	
}


function setStr(val) {
	generalStrength = val;
	for(var i=0; i<constraints.length; i++) {
		constraints[i].strength = val;
	}
}
function setTau(val) {
	generalTau = val;
	for(var i=0; i<constraints.length; i++) {
		if(constraints[i] instanceof AngularDashpot) {		
			constraints[i].damping = val;
		} else {
			constraints[i].tau = val;
		}
	}
}

Button("=============================").add();
Button("  Constraints ").add();
Button("=============================").add();

this.prismaticTest = Button("Prismatic Constraint").add();
this.prismaticTest.parent=this;
this.prismaticTest.onClick = function() {
	this.parent.doPrismaticConstraint(boxa, boxb, solver);
}
this.stiffSpringTest = Button("Stiff Spring Constraint").add();
this.stiffSpringTest.parent=this;
this.stiffSpringTest.onClick = function() {
	this.parent.doStiffSpringConstraint(boxa, boxb, solver);
}
this.ptpTest = Button("Point to Point Constraint").add();
this.ptpTest.parent=this;
this.ptpTest.onClick = function() {
	this.parent.doPointToPointConstraint(boxa, boxb, solver);
}
this.aDashpotTest = Button("Angular Dashpot Test").add();
this.aDashpotTest.parent=this;
this.aDashpotTest.onClick = function() {
	this.parent.doAngularDashpot(boxa, boxb, solver);
}
this.hingeTest = Button("Hinge Constraint").add();
this.hingeTest.parent=this;
this.hingeTest.onClick = function() {
	this.parent.doHingeConstraint(boxa, boxb, solver);
}
Button("==============================").add();
Button("  Actions").add();
Button("=============================").add();
this.magneticActionTest = Button("Magnetic Action").add();
this.magneticActionTest.parent = this;
this.magneticActionTest.onClick = function() {
	this.parent.doMagneticAction(boxa, boxb, solver);
}
this.simpleRigidWindActionTest = Button("Simple Wind Action").add();
this.simpleRigidWindActionTest.parent = this;
this.simpleRigidWindActionTest.onClick = function() {
	this.parent.doSimpleRigidWindAction(boxa, boxb, solver);
}
Button("==============================").add();
this.clearConstraintBtn = Button("Clear Constraints").add();
this.clearConstraintBtn.parent=this;
this.clearConstraintBtn.onClick = function() {
	this.parent.clearConstraints();
}
this.clearActionBtn = Button("Clear Actions").add();
this.clearActionBtn.parent=this;
this.clearActionBtn.onClick = function() {
	this.parent.clearActions();
}
this.resetBoxesBtn = Button("Reset Everything").add();
this.resetBoxesBtn.parent=this;
this.resetBoxesBtn.onClick = function() {
	this.parent.clearConstraints();
	this.parent.clearActions();
	boxa.reset();	
	boxb.reset();
}

